//
// Copyright 2014 ADVANCED MICRO DEVICES, INC.  All Rights Reserved.
//
// AMD is granting you permission to use this software and documentation (if
// any) (collectively, the "Materials") pursuant to the terms and conditions
// of the Software License Agreement included with the Materials.  If you do
// not have a copy of the Software License Agreement, contact your AMD
// representative for a copy.
// You agree that you will not reverse engineer or decompile the Materials,
// in whole or in part, except as allowed by applicable law.
//
// WARRANTY DISCLAIMER: THE SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND.  AMD DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, OR STATUTORY,
// INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE, NON-INFRINGEMENT, THAT THE SOFTWARE
// WILL RUN UNINTERRUPTED OR ERROR-FREE OR WARRANTIES ARISING FROM CUSTOM OF
// TRADE OR COURSE OF USAGE.  THE ENTIRE RISK ASSOCIATED WITH THE USE OF THE
// SOFTWARE IS ASSUMED BY YOU.
// Some jurisdictions do not allow the exclusion of implied warranties, so
// the above exclusion may not apply to You. 
// 
// LIMITATION OF LIABILITY AND INDEMNIFICATION:  AMD AND ITS LICENSORS WILL
// NOT, UNDER ANY CIRCUMSTANCES BE LIABLE TO YOU FOR ANY PUNITIVE, DIRECT,
// INCIDENTAL, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING FROM USE OF
// THE SOFTWARE OR THIS AGREEMENT EVEN IF AMD AND ITS LICENSORS HAVE BEEN
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.  
// In no event shall AMD's total liability to You for all damages, losses,
// and causes of action (whether in contract, tort (including negligence) or
// otherwise) exceed the amount of $100 USD.  You agree to defend, indemnify
// and hold harmless AMD and its licensors, and any of their directors,
// officers, employees, affiliates or agents from and against any and all
// loss, damage, liability and other expenses (including reasonable attorneys'
// fees), resulting from Your use of the Software or violation of the terms and
// conditions of this Agreement.  
//
// U.S. GOVERNMENT RESTRICTED RIGHTS: The Materials are provided with "RESTRICTED
// RIGHTS." Use, duplication, or disclosure by the Government is subject to the
// restrictions as set forth in FAR 52.227-14 and DFAR252.227-7013, et seq., or
// its successor.  Use of the Materials by the Government constitutes
// acknowledgement of AMD's proprietary rights in them.
// 
// EXPORT RESTRICTIONS: The Materials may be subject to export restrictions as
// stated in the Software License Agreement.
//
#include "Globals.h"


// A texture filled with random values for generating some variance in our particles when we spawn them
Texture2D								g_RandomBuffer			: register( t0 );

// The particle buffers to fill with new particles
RWStructuredBuffer<GPUParticlePartA>	g_ParticleBufferA		: register( u0 );
RWStructuredBuffer<GPUParticlePartB>	g_ParticleBufferB		: register( u1 );

// The dead list interpretted as a consume buffer. So every time we consume an index from this list, it automatically decrements the atomic counter (ie the number of dead particles)
ConsumeStructuredBuffer<uint>			g_DeadListToAllocFrom	: register( u2 );


cbuffer EmitterConstantBuffer : register( b1 )
{
	float4	g_vEmitterPosition;
	float4	g_vEmitterVelocity;
	float4	g_PositionVariance;
	
	int		g_MaxParticlesThisFrame;
	float	g_ParticleLifeSpan;	
	float	g_StartSize;
	float	g_EndSize;
	
	float	g_VelocityVariance;
	float	g_Mass;
	uint	g_EmitterIndex;
	uint	g_EmitterStreaks;

	uint	g_TextureIndex;
	uint	g_pads[ 3 ];
};


// Emit particles, one per thread, in blocks of 1024 at a time
[numthreads(1024,1,1)]
void CS_Emit( uint3 id : SV_DispatchThreadID )
{
	// Check to make sure we don't emit more particles than we specified
	if ( id.x < g_NumDeadParticles && id.x < g_MaxParticlesThisFrame )
	{
		// Initialize the particle data to zero to avoid any unexpected results
		GPUParticlePartA pa = (GPUParticlePartA)0;
		GPUParticlePartB pb = (GPUParticlePartB)0;
		
		// Generate some random numbers from reading the random texture
		float2 uv = float2( id.x / 1024.0, g_ElapsedTime );
		float3 randomValues0 = g_RandomBuffer.SampleLevel( g_samWrapLinear, uv, 0 ).xyz;

		float2 uv2 = float2( (id.x + 1) / 1024.0, g_ElapsedTime );
		float3 randomValues1 = g_RandomBuffer.SampleLevel( g_samWrapLinear, uv2, 0 ).xyz;

		float velocityMagnitude = length( g_vEmitterVelocity.xyz );

		pb.m_Position = g_vEmitterPosition.xyz + ( randomValues0.xyz * g_PositionVariance.xyz );

		pa.m_EmitterProperties = WriteEmitterProperties( g_EmitterIndex, g_TextureIndex, g_EmitterStreaks ? true : false );
		pa.m_Rotation = 0;
		pa.m_IsSleeping = 0;
		pa.m_CollisionCount = 0;

		pb.m_Mass = g_Mass;
		pb.m_Velocity = g_vEmitterVelocity.xyz + ( randomValues1.xyz * velocityMagnitude * g_VelocityVariance );
		pb.m_Lifespan = g_ParticleLifeSpan;
		pb.m_Age = pb.m_Lifespan;
		pb.m_StartSize = g_StartSize;
		pb.m_EndSize = g_EndSize;

		// The index into the global particle list obtained from the dead list. 
		// Calling consume will decrement the counter in this buffer.
		uint index = g_DeadListToAllocFrom.Consume();

		// Write the new particle state into the global particle buffer
		g_ParticleBufferA[ index ] = pa;
		g_ParticleBufferB[ index ] = pb;
	}
}