//
// Copyright 2014 ADVANCED MICRO DEVICES, INC.  All Rights Reserved.
//
// AMD is granting you permission to use this software and documentation (if
// any) (collectively, the "Materials") pursuant to the terms and conditions
// of the Software License Agreement included with the Materials.  If you do
// not have a copy of the Software License Agreement, contact your AMD
// representative for a copy.
// You agree that you will not reverse engineer or decompile the Materials,
// in whole or in part, except as allowed by applicable law.
//
// WARRANTY DISCLAIMER: THE SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND.  AMD DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, OR STATUTORY,
// INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE, NON-INFRINGEMENT, THAT THE SOFTWARE
// WILL RUN UNINTERRUPTED OR ERROR-FREE OR WARRANTIES ARISING FROM CUSTOM OF
// TRADE OR COURSE OF USAGE.  THE ENTIRE RISK ASSOCIATED WITH THE USE OF THE
// SOFTWARE IS ASSUMED BY YOU.
// Some jurisdictions do not allow the exclusion of implied warranties, so
// the above exclusion may not apply to You. 
// 
// LIMITATION OF LIABILITY AND INDEMNIFICATION:  AMD AND ITS LICENSORS WILL
// NOT, UNDER ANY CIRCUMSTANCES BE LIABLE TO YOU FOR ANY PUNITIVE, DIRECT,
// INCIDENTAL, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING FROM USE OF
// THE SOFTWARE OR THIS AGREEMENT EVEN IF AMD AND ITS LICENSORS HAVE BEEN
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.  
// In no event shall AMD's total liability to You for all damages, losses,
// and causes of action (whether in contract, tort (including negligence) or
// otherwise) exceed the amount of $100 USD.  You agree to defend, indemnify
// and hold harmless AMD and its licensors, and any of their directors,
// officers, employees, affiliates or agents from and against any and all
// loss, damage, liability and other expenses (including reasonable attorneys'
// fees), resulting from Your use of the Software or violation of the terms and
// conditions of this Agreement.  
//
// U.S. GOVERNMENT RESTRICTED RIGHTS: The Materials are provided with "RESTRICTED
// RIGHTS." Use, duplication, or disclosure by the Government is subject to the
// restrictions as set forth in FAR 52.227-14 and DFAR252.227-7013, et seq., or
// its successor.  Use of the Materials by the Government constitutes
// acknowledgement of AMD's proprietary rights in them.
// 
// EXPORT RESTRICTIONS: The Materials may be subject to export restrictions as
// stated in the Software License Agreement.
//

#include "Globals.h"


struct VS_RenderSceneInput
{
    float3 f3Position	: POSITION;  
    float3 f3Normal		: NORMAL;     
    float2 f2TexCoord	: TEXCOORD0;
	float3 f3Tangent	: TANGENT;    
};

struct PS_RenderSceneInput
{
    float4 f4Position   : SV_Position;
	float2 f2TexCoord	: TEXCOORD0;
	float3 f3Normal     : NORMAL; 
	float3 f3Tangent    : TANGENT;
	float3 f3WorldPos	: TEXCOORD2;	
};

Texture2D g_txDiffuse	: register( t2 );
Texture2D g_txNormal	: register( t3 );

//=================================================================================================================================
// This shader computes standard transform and lighting
//=================================================================================================================================
PS_RenderSceneInput VS_RenderScene( VS_RenderSceneInput I )
{
    PS_RenderSceneInput O;
     
    // Transform the position from object space to homogeneous projection space
    O.f4Position = mul( float4( I.f3Position, 1.0f ), g_mViewProjection );
    
    // Transform the normal, tangent and position from object space to world space    
    //O.f3WorldPos = mul( I.f3Position, (float3x3)g_f4x4World );
    //O.f3Normal  = normalize( mul( I.f3Normal, (float3x3)g_f4x4World ) );
	//O.f3Tangent = normalize( mul( I.f3Tangent, (float3x3)g_f4x4World ) );
    O.f3WorldPos = I.f3Position;
    O.f3Normal  = normalize( I.f3Normal );
	O.f3Tangent = normalize( I.f3Tangent );
    
	// Pass through tex coords
	O.f2TexCoord = I.f2TexCoord;
    
    return O;    
}


//=================================================================================================================================
// This shader outputs the pixel's color by passing through the lit 
// diffuse material color
//=================================================================================================================================
float4 PS_RenderScene( PS_RenderSceneInput I ) : SV_Target0
{
	float4 f4Diffuse = g_txDiffuse.Sample( g_samWrapLinear, I.f2TexCoord );
    float fSpecMask = f4Diffuse.a;
    float3 f3Norm = g_txNormal.Sample( g_samWrapLinear, I.f2TexCoord ).xyz;
    f3Norm *= 2.0f;
    f3Norm -= float3( 1.0f, 1.0f, 1.0f );
    
    float3 f3Binorm = normalize( cross( I.f3Normal, I.f3Tangent ) );
    float3x3 f3x3BasisMatrix = float3x3( f3Binorm, I.f3Tangent, I.f3Normal );
    f3Norm = normalize( mul( f3Norm, f3x3BasisMatrix ) );
   
    // Diffuse lighting
    float4 f4Lighting = saturate( dot( f3Norm, g_SunDirection.xyz ) ) * g_SunColor;
    f4Lighting += g_AmbientColor;
    
    // Calculate specular power
    float3 f3ViewDir = normalize( g_EyePosition.xyz - I.f3WorldPos );
    float3 f3HalfAngle = normalize( f3ViewDir + g_SunDirection.xyz );
    float4 f4SpecPower1 = pow( saturate( dot( f3HalfAngle, f3Norm ) ), 32 ) * g_SunColor;
    
    return f4Lighting * f4Diffuse + ( f4SpecPower1 * fSpecMask );
}



//--------------------------------------------------------------------------------------
// PS for the sky
//--------------------------------------------------------------------------------------
float4 PS_Sky( PS_RenderSceneInput I ) : SV_Target
{
    float4 f4O;

    // Bog standard textured rendering
    f4O.xyz = g_txDiffuse.Sample( g_samWrapLinear, I.f2TexCoord ).xyz;
    f4O.w = 1.0f;

    return f4O;
}